@@ -4,7 +4,7 @@ Attean - A Semantic Web Framework
 
 =head1 VERSION
 
-This document describes Attean version 0.033	
+This document describes Attean version 0.035
 
 =head1 SYNOPSIS
 
@@ -51,7 +51,7 @@ Semantic Web (RDF and SPARQL) data.
 package Attean {
 	use v5.14;
 	use warnings;
-	our $VERSION	= '0.033';
+	our $VERSION	= '0.035';
 	use Attean::API;
 	
 	use Attean::Blank;
@@ -76,7 +76,7 @@ package Attean {
 	use Attean::TermMap;
 	
 	use HTTP::Negotiate qw(choose);
-	use List::MoreUtils qw(any all);
+	use List::Util qw(any all);
 	use Module::Load::Conditional qw(can_load);
 	use Role::Tiny ();
 	use Sub::Util qw(set_subname);
@@ -431,55 +431,116 @@ currently undefined.
 	}
 	
 	
-	our %global_functions;
+	# Global registry for extension functions that can be invoked via BIND, FILTER, etc.
+	{
+		our %global_functions;
 
 =item C<< register_global_function( %uri_to_func ) >>
 
 =cut
-	sub register_global_function {
-		my $class	= shift;
-		my %args	= @_;
-		foreach my $uri (keys %args) {
-			my $func	= $args{ $uri };
-			$global_functions{ $uri }	= $func;
+		sub register_global_function {
+			my $class	= shift;
+			my %args	= @_;
+			foreach my $uri (keys %args) {
+				my $func	= $args{ $uri };
+				$global_functions{ $uri }	= $func;
+			}
 		}
-	}
 
 =item C<< get_global_function( $uri ) >>
 
 =cut
-	sub get_global_function {
-		my $class	= shift;
-		my $uri		= shift;
-		return $global_functions{ $uri };
+		sub get_global_function {
+			my $class	= shift;
+			my $uri		= shift;
+			return $global_functions{ $uri };
+		}
 	}
 	
-	our %global_aggregates;
+	# Global registry for extension "functional forms" that can be invoked via BIND, FILTER, etc.
+	# These differ from extension "functions" in that they can be passed undef values as arguments
+	# for expressions whose evaluation resulted in an error.
+	{
+		our %global_functional_forms;
+
+=item C<< register_global_functional_form( %uri_to_func ) >>
+
+=cut
+		sub register_global_functional_form {
+			my $class	= shift;
+			my %args	= @_;
+			foreach my $uri (keys %args) {
+				my $func	= $args{ $uri };
+				$global_functional_forms{ $uri }	= $func;
+			}
+		}
+
+=item C<< get_global_functional_form( $uri ) >>
+
+=cut
+		sub get_global_functional_form {
+			my $class	= shift;
+			my $uri		= shift;
+			return $global_functional_forms{ $uri };
+		}
+	}
+	
+	# Global registry for extension aggregates
+	{
+		our %global_aggregates;
 
 =item C<< register_global_aggregate( %uri_to_hash ) >>
 
 =cut
-	sub register_global_aggregate {
-		my $class	= shift;
-		my %args	= @_;
-		foreach my $uri (keys %args) {
-			my $funcs	= $args{ $uri };
-			$global_aggregates{ $uri }	= $funcs;
+		sub register_global_aggregate {
+			my $class	= shift;
+			my %args	= @_;
+			foreach my $uri (keys %args) {
+				my $funcs	= $args{ $uri };
+				$global_aggregates{ $uri }	= $funcs;
+			}
 		}
-	}
 
 =item C<< get_global_aggregate( $uri ) >>
 
 =cut
-	sub get_global_aggregate {
-		my $class	= shift;
-		my $uri		= shift;
-		return $global_aggregates{ $uri };
+		sub get_global_aggregate {
+			my $class	= shift;
+			my $uri		= shift;
+			return $global_aggregates{ $uri };
+		}
 	}
 	
-	
+	# Global registry for extension datatypes.
+	# When literals of this datatype are constructed, they will have the registered Moo role applied to them.
+	{
+		our %datatype_roles;
+
+=item C<< register_datatype_role( %uri_to_role ) >>
+
+=cut
+		sub register_datatype_role {
+			my $class	= shift;
+			my %args	= @_;
+			foreach my $uri (keys %args) {
+				my $func	= $args{ $uri };
+				$datatype_roles{ $uri }	= $func;
+			}
+		}
+
+=item C<< get_datatype_role( $uri ) >>
+
+=cut
+		sub get_datatype_role {
+			my $class	= shift;
+			my $uri		= shift;
+			return $datatype_roles{ $uri };
+		}
+	}
 }
 
+use AtteanX::Functions::CompositeLists;
+
 1;
 
 __END__
