@@ -7,7 +7,7 @@ Attean::API::Parser - Parser role
 
 =head1 VERSION
 
-This document describes Attean::API::Parser version 0.021
+This document describes Attean::API::Parser version 0.035
 
 =head1 DESCRIPTION
 
@@ -48,14 +48,21 @@ to this parser.
 Returns a L<Type::Tiny> object representing the type of items that result from
 parsing.
 
+=item C<< file_extensions >>
+
+Returns an ARRAY reference of file extensions commonly associated with the
+media types supported by the parser (and returned by C<< media_types >>).
+File extensions should NOT include a leading dot.
+
 =cut
 
 use Type::Tiny::Role;
 
-package Attean::API::Parser 0.021 {
+package Attean::API::Parser 0.035 {
 	use Types::Standard qw(CodeRef Bool);
 
 	use Moo::Role;
+	use Scalar::Util qw(blessed);
 	use namespace::clean;
 	
 	has 'handler' => (is => 'rw', isa => CodeRef, default => sub { sub {} });
@@ -87,21 +94,73 @@ C<lazy_iris> attribute.
 		}
 		return Attean::IRI->new(%args);
 	}
+
+=item C<< new_literal( value => $value, [ datatype => $dt, ] [ language => $lang ])	>>
+
+Constructs and returns a new L<Attean::Literal> object.
+
+=cut
+
+	sub new_literal {
+		my $self	= shift;
+		my %args	= @_;
+		
+		return Attean::Literal->new(%args);
+	}
+	
+	sub file_extensions { return [] }
 }
 
-package Attean::API::AbbreviatingParser 0.021 {
+package Attean::API::AbbreviatingParser 0.035 {
 	use Types::Standard qw(ConsumerOf InstanceOf Maybe);
-	use URI::NamespaceMap;
+	use Types::Namespace qw( NamespaceMap );
 	use Scalar::Util qw(blessed);
 
 	use Moo::Role;
 	
 	with 'Attean::API::Parser';
-	has 'base' 		=> (is => 'rw', isa => ConsumerOf['Attean::API::IRI'], coerce => sub { blessed($_[0]) ? Attean::IRI->new($_[0]->as_string) : Attean::IRI->new($_[0]) }, predicate => 'has_base');
-	has 'namespaces'	=> (is => 'ro', isa => Maybe[InstanceOf['URI::NamespaceMap']]);
+	has 'base' 			=> (is => 'rw', isa => ConsumerOf['Attean::API::IRI'], coerce => sub { blessed($_[0]) ? Attean::IRI->new($_[0]->as_string) : Attean::IRI->new($_[0]) }, predicate => 'has_base');
+	has 'namespaces'	=> (is => 'ro', isa => Maybe[NamespaceMap]);
 }
 
-package Attean::API::PushParser 0.021 {
+package Attean::API::CDTBlankNodeMappingParser 0.035 {
+	use Types::Standard qw(HashRef ConsumerOf InstanceOf Maybe Bool);
+	use Scalar::Util qw(blessed);
+	use UUID::Tiny ':std';
+	use Data::Dumper;
+	use Moo::Role;
+	
+	with 'Attean::API::Parser';
+	has 'blank_nodes'	=> (is => 'ro', isa => Maybe[HashRef[ConsumerOf['Attean::API::Blank']]]);
+	has 'parse_id'		=> (is => 'rw', default => sub { unpack('H*', create_uuid()) });
+	has 'enable_cdt_rewriting' => (is => 'rw', isa => Bool, default => 1);
+
+	foreach my $method (qw(parse_iter_from_io parse_iter_from_bytes parse_cb_from_io parse_cb_from_bytes)) {
+		around $method => sub {
+			my $orig	= shift;
+			my $self	= $_[0];
+
+			$self->parse_id(unpack('H*', create_uuid()));
+			my $term	= $orig->(@_);
+			return $term;
+		};
+	}
+	
+	around 'new_literal' => sub {
+		my $orig	= shift;
+		my $self	= $_[0];
+		my $term	= $orig->(@_);
+		if ($self->enable_cdt_rewriting) {
+			my $dt 		= $term->datatype();
+			if (blessed($dt) and ($dt->value eq 'http://w3id.org/awslabs/neptune/SPARQL-CDTs/Map' or $dt->value eq 'http://w3id.org/awslabs/neptune/SPARQL-CDTs/List')) {
+				return AtteanX::Functions::CompositeLists::rewrite_lexical($term, $self->blank_nodes, $self->parse_id);
+			}
+		}
+		return $term;
+	};
+}
+
+package Attean::API::PushParser 0.035 {
 	use Moo::Role;
 	with 'Attean::API::Parser';
 
@@ -165,7 +224,7 @@ package Attean::API::PushParser 0.021 {
 	}
 }
 
-package Attean::API::PullParser 0.021 {
+package Attean::API::PullParser 0.035 {
 	use Moo::Role;
 	with 'Attean::API::Parser';
 	
@@ -207,7 +266,7 @@ package Attean::API::PullParser 0.021 {
 	}
 }
 
-package Attean::API::AtOnceParser 0.021 {
+package Attean::API::AtOnceParser 0.035 {
 	use Moo::Role;
 	with 'Attean::API::Parser';
 	
@@ -268,7 +327,7 @@ package Attean::API::AtOnceParser 0.021 {
 	
 }
 
-package Attean::API::TermParser 0.021 {
+package Attean::API::TermParser 0.035 {
 	# Parser returns objects that conform to Attean::API::Term
 	use Moo::Role;
 	with 'Attean::API::Parser';
@@ -278,7 +337,7 @@ package Attean::API::TermParser 0.021 {
 	}
 }
 
-package Attean::API::TripleParser 0.021 {
+package Attean::API::TripleParser 0.035 {
 	# Parser returns objects that conform to Attean::API::Triple
 	use Moo::Role;
 	with 'Attean::API::Parser';
@@ -288,7 +347,7 @@ package Attean::API::TripleParser 0.021 {
 	}
 }
 
-package Attean::API::QuadParser 0.021 {
+package Attean::API::QuadParser 0.035 {
 	# Parser returns objects that conform to Attean::API::Quad
 	use Moo::Role;
 	with 'Attean::API::Parser';
@@ -298,7 +357,7 @@ package Attean::API::QuadParser 0.021 {
 	}
 }
 
-package Attean::API::MixedStatementParser 0.021 {
+package Attean::API::MixedStatementParser 0.035 {
 	# Parser returns objects that conform to either Attean::API::Triple or Attean::API::Quad
 	use Moo::Role;
 	with 'Attean::API::Parser';
@@ -308,7 +367,7 @@ package Attean::API::MixedStatementParser 0.021 {
 	}
 }
 
-package Attean::API::ResultOrTermParser 0.021 {
+package Attean::API::ResultOrTermParser 0.035 {
 	# Parser returns objects that conform to either Attean::API::Result or Attean::API::Term
 	use Moo::Role;
 	with 'Attean::API::Parser';
@@ -318,7 +377,7 @@ package Attean::API::ResultOrTermParser 0.021 {
 	}
 }
 
-package Attean::API::ResultParser 0.021 {
+package Attean::API::ResultParser 0.035 {
 	# Parser returns objects that conform to Attean::API::Result
 	use Moo::Role;
 	with 'Attean::API::Parser';
@@ -349,7 +408,7 @@ Gregory Todd Williams  C<< <gwilliams@cpan.org> >>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2014--2018 Gregory Todd Williams.
+Copyright (c) 2014--2022 Gregory Todd Williams.
 This program is free software; you can redistribute it and/or modify it under
 the same terms as Perl itself.
 
