@@ -7,7 +7,7 @@ Attean::API::IDPJoinPlanner - Iterative dynamic programming query planning role
 
 =head1 VERSION
 
-This document describes Attean::API::IDPJoinPlanner version 0.030
+This document describes Attean::API::IDPJoinPlanner version 0.035
 
 =head1 SYNOPSIS
 
@@ -32,15 +32,16 @@ methods that consume the L<Attean::API::CostPlanner> role.
 
 =cut
 
-package Attean::API::QueryPlanner 0.030 {
-	use Types::Standard qw(CodeRef);
+package Attean::API::QueryPlanner 0.035 {
+	use Types::Standard qw(CodeRef Object);
 
 	use Moo::Role;
 	
 	requires 'plan_for_algebra'; # plan_for_algebra($algebra, $model, \@default_graphs)
+	has 'request_signer' => (is => 'rw', isa => Object);
 }
 
-package Attean::API::CostPlanner 0.030 {
+package Attean::API::CostPlanner 0.035 {
 	use Scalar::Util qw(refaddr);
 	use Types::Standard qw(CodeRef);
 
@@ -75,12 +76,12 @@ package Attean::API::CostPlanner 0.030 {
 	}
 }
 
-package Attean::API::JoinPlanner 0.030 {
+package Attean::API::JoinPlanner 0.035 {
 	use Moo::Role;
 	requires 'joins_for_plan_alternatives';
 }
 
-package Attean::API::NaiveJoinPlanner 0.030 {
+package Attean::API::NaiveJoinPlanner 0.035 {
 	use Math::Cartesian::Product;
 
 	use Moo::Role;
@@ -108,7 +109,7 @@ package Attean::API::NaiveJoinPlanner 0.030 {
 	}
 }
 
-package Attean::API::SimpleCostPlanner 0.030 {
+package Attean::API::SimpleCostPlanner 0.035 {
 	use Types::Standard qw(Int);
 	use Scalar::Util qw(blessed);
 
@@ -213,13 +214,13 @@ package Attean::API::SimpleCostPlanner 0.030 {
 	}
 }
 
-package Attean::API::IDPJoinPlanner 0.030 {
+package Attean::API::IDPJoinPlanner 0.035 {
 	use Encode qw(encode);
 	use Attean::RDF;
 	use LWP::UserAgent;
 	use Scalar::Util qw(blessed reftype);
 	use List::Util qw(reduce);
-	use List::MoreUtils qw(all any);
+	use List::Util qw(all any);
 	use Types::Standard qw(Int ConsumerOf InstanceOf);
 	use URI::Escape;
 	use Algorithm::Combinatorics qw(subsets);
@@ -562,7 +563,7 @@ Gregory Todd Williams  C<< <gwilliams@cpan.org> >>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2014--2020 Gregory Todd Williams.
+Copyright (c) 2014--2022 Gregory Todd Williams.
 This program is free software; you can redistribute it and/or modify it under
 the same terms as Perl itself.
 
