@@ -4,7 +4,7 @@ Attean::RDF - Utility package for exporting shorthand functions for constructing
 
 =head1 VERSION
 
-This document describes Attean::RDF version 0.026
+This document describes Attean::RDF version 0.035
 
 =head1 SYNOPSIS
 
@@ -31,18 +31,18 @@ by default).
 
 =cut
 
-package Attean::RDF 0.026 {
+package Attean::RDF 0.035 {
 	use v5.14;
 	use warnings;
 
-	use Attean;
-	use List::MoreUtils qw(zip);
 	require Exporter::Tiny;
-	use namespace::clean;
-
 	our @ISA	= qw(Exporter::Tiny);
 	our @EXPORT	= qw(iri blank literal dtliteral langliteral variable triple quad triplepattern quadpattern bgp);
 
+	require Attean;
+	use List::Util qw(mesh);
+	use namespace::clean;
+
 =item C<< variable( $value ) >>
 
 C<< Attean::Variable->new($value) >>
@@ -91,7 +91,7 @@ C<< Attean::Literal->new( value => $value, datatype => $dt ) >>
 
 	sub dtliteral {
 		my @k	= qw(value datatype);
-		return Attean::Literal->new(zip @k, @_);
+		return Attean::Literal->new(mesh \@k, \@_);
 	}
 
 =item C<< langliteral( $value, $lang ) >>
@@ -102,7 +102,8 @@ C<< Attean::Literal->new( value => $value, language => $lang ) >>
 
 	sub langliteral {
 		my @k	= qw(value language);
-		return Attean::Literal->new(zip @k, @_);
+		$DB::single = 1;
+		return Attean::Literal->new(mesh \@k, \@_);
 	}
 	
 =item C<< triple( @terms ) >>
@@ -181,7 +182,7 @@ Gregory Todd Williams  C<< <gwilliams@cpan.org> >>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2014--2019 Gregory Todd Williams.
+Copyright (c) 2014--2022 Gregory Todd Williams.
 This program is free software; you can redistribute it and/or modify it under
 the same terms as Perl itself.
 
