@@ -7,7 +7,7 @@ AtteanX::Parser::SPARQLLex - SPARQL Lexer
 
 =head1 VERSION
 
-This document describes AtteanX::Parser::SPARQLLex version 0.021
+This document describes AtteanX::Parser::SPARQLLex version 0.035
 
 =head1 SYNOPSIS
 
@@ -39,13 +39,12 @@ This document describes AtteanX::Parser::SPARQLLex version 0.021
 
 =cut
 
-package AtteanX::Parser::SPARQLLex 0.021 {
+package AtteanX::Parser::SPARQLLex 0.035 {
 	use utf8;
 	use Moo;
 	use Attean;
 	use Encode;
 	use Encode qw(decode);
-	use List::MoreUtils qw(zip);
 	use Types::Standard qw(ArrayRef);
 	use namespace::clean;
 
@@ -104,13 +103,12 @@ the SPARQL query/update read from the L<IO::Handle> object C<< $fh >>.
 	}
 }
 
-package AtteanX::Parser::SPARQLLex::Iterator 0.021 {
+package AtteanX::Parser::SPARQLLex::Iterator 0.035 {
 	use utf8;
 	use Moo;
 	use Attean;
 	use Encode;
 	use Encode qw(decode);
-	use List::MoreUtils qw(zip);
 	use AtteanX::SPARQL::Token;
 	use AtteanX::SPARQL::Constants;
 	use Types::Standard qw(FileHandle Ref Str Int ArrayRef HashRef ConsumerOf InstanceOf);
@@ -149,7 +147,7 @@ package AtteanX::Parser::SPARQLLex::Iterator 0.021 {
 	my $r_BLANK_NODE_LABEL		= qr/_:${r_PN_LOCAL_BNODE}/o;
 	my $r_ANON					= qr/\[[\t\r\n ]*\]/o;
 	my $r_NIL					= qr/\([\n\r\t ]*\)/o;
-	my $r_KEYWORDS				= qr/(ABS|ADD|ALL|ASC|ASK|AS|AVG|BASE|BIND|BNODE|BOUND|BY|CEIL|CLEAR|COALESCE|CONCAT|CONSTRUCT|CONTAINS|COPY|COUNT|CREATE|DATATYPE|DAY|DEFAULT|DELETE|DELETE WHERE|DESCRIBE|DESC|DISTINCT|DISTINCT|DROP|ENCODE_FOR_URI|EXISTS|FILTER|FLOOR|FROM|GRAPH|GROUP_CONCAT|GROUP|HAVING|HOURS|IF|INSERT|INSERT|DATA|INTO|IN|IRI|ISBLANK|ISIRI|ISLITERAL|ISNUMERIC|ISURI|LANGMATCHES|LANG|LCASE|LIMIT|LOAD|MAX|MD5|MINUS|MINUTES|MIN|MONTH|MOVE|NAMED|NOT|NOW|OFFSET|OPTIONAL|ORDER|PREFIX|RAND|REDUCED|REGEX|REPLACE|ROUND|SAMETERM|SAMPLE|SECONDS|SELECT|SEPARATOR|SERVICE|SHA1|SHA256|SHA384|SHA512|SILENT|STRAFTER|STRBEFORE|STRDT|STRENDS|STRLANG|STRLEN|STRSTARTS|STRUUID|STR|SUBSTR|SUM|TIMEZONE|TO|TZ|UCASE|UNDEF|UNION|URI|USING|UUID|VALUES|WHERE|WITH|YEAR)(?!:)\b/io;
+	my $r_KEYWORDS				= qr/(ABS|ADD|ALL|ASC|ASK|AS|AVG|BASE|BIND|BNODE|BOUND|BY|CEIL|CLEAR|COALESCE|CONCAT|CONSTRUCT|CONTAINS|COPY|COUNT|CREATE|DATATYPE|DAY|DEFAULT|DELETE|DELETE WHERE|DESCRIBE|DESC|DISTINCT|DISTINCT|DROP|ENCODE_FOR_URI|EXISTS|FILTER|FLOOR|FROM|GRAPH|GROUP_CONCAT|GROUP|HAVING|HOURS|IF|INSERT|INSERT|DATA|INTO|IN|IRI|ISBLANK|ISIRI|ISLITERAL|ISNUMERIC|ISURI|LANGMATCHES|LANG|LCASE|LIMIT|LOAD|MAX|MD5|MINUS|MINUTES|MIN|MONTH|MOVE|NAMED|NOT|NOW|OFFSET|OPTIONAL|ORDER|PREFIX|RAND|REDUCED|REGEX|REPLACE|ROUND|SAMETERM|SAMPLE|SECONDS|SELECT|SEPARATOR|SERVICE|SHA1|SHA256|SHA384|SHA512|SILENT|STRAFTER|STRBEFORE|STRDT|STRENDS|STRLANG|STRLEN|STRSTARTS|STRUUID|STR|SUBSTR|SUM|TIMEZONE|TO|TZ|UCASE|UNDEF|UNION|URI|USING|UUID|VALUES|WHERE|WITH|YEAR|TRIPLE|ISTRIPLE|SUBJECT|PREDICATE|OBJECT|HINT|UNFOLD|FOLD)(?!:)\b/io;
 
 	sub BUILD {
 		my $self	= shift;
@@ -157,11 +155,15 @@ package AtteanX::Parser::SPARQLLex::Iterator 0.021 {
 		# 	q[#]	=> '_get_comment',
 			q[@]	=> '_get_lang',
 			q[<]	=> '_get_iriref_or_relational',
+			q[{]	=> '_get_brace_or_annotation_or_or',
+			q[}]	=> '_get_brace_or_annotation_or_or',
+			q[|]	=> '_get_brace_or_annotation_or_or',
 			q[_]	=> '_get_bnode',
 			q[']	=> '_get_single_literal',
 			q["]	=> '_get_double_literal',
 			q[:]	=> '_get_pname',
 			q[?]	=> '_get_variable',
+			q[$]	=> '_get_variable',
 			q[!]	=> '_get_bang',
 			q[>]	=> '_get_iriref_or_relational',
 			q([)	=> '_get_lbracket_or_anon',
@@ -179,8 +181,6 @@ package AtteanX::Parser::SPARQLLex::Iterator 0.021 {
 			','	=> COMMA,
 			'.'	=> DOT,
 			'='	=> EQUALS,
-			'{'	=> LBRACE,
-			'}'	=> RBRACE,
 			']'	=> RBRACKET,
 			')'	=> RPAREN,
 			'-'	=> MINUS,
@@ -223,8 +223,17 @@ package AtteanX::Parser::SPARQLLex::Iterator 0.021 {
 		unless (length($self->buffer)) {
 			my $line	= $self->file->getline;
 			if (defined($line)) {
-				$line		=~ s/\\u([0-9A-Fa-f]{4})/chr(hex($1))/ge;
-				$line		=~ s/\\U([0-9A-Fa-f]{8})/chr(hex($1))/ge;
+				no warnings 'uninitialized';
+				$line		=~ s{\\(?:(?:u([0-9A-Fa-f]{4}))|(?:U([0-9A-Fa-f]{8})))}{
+					my $h			= $1 . $2;
+					my $codepoint	= hex($h);
+					if ($codepoint >= 0xD800 and $codepoint <= 0xDFFF) {
+						die "Unicode surrogate U+$h is illegal in UTF-8";
+					}
+					chr($codepoint);
+				}ge;
+# 				$line		=~ s/\\u([0-9A-Fa-f]{4})/chr(hex($1))/ge;
+# 				$line		=~ s/\\U([0-9A-Fa-f]{8})/chr(hex($1))/ge;
 				$self->{buffer}	.= $line;
 			}
 		}
@@ -323,14 +332,6 @@ package AtteanX::Parser::SPARQLLex::Iterator 0.021 {
 					$self->read_word('^');
 					return $self->new_token(HAT, $start_line, $start_column, '^');
 				}
-			} elsif ($c eq '|') {
-				if ($self->buffer =~ /^\|\|/) {
-					$self->read_word('||');
-					return $self->new_token(OROR, $start_line, $start_column, '||');
-				} else {
-					$self->read_word('|');
-					return $self->new_token(OR, $start_line, $start_column, '|');
-				}
 			} elsif ($c eq '&') {
 				$self->read_word('&&');
 				return $self->new_token(ANDAND, $start_line, $start_column, '&&');
@@ -362,12 +363,22 @@ package AtteanX::Parser::SPARQLLex::Iterator 0.021 {
 
 	sub _get_variable {
 		my $self	= shift;
-		$self->get_char_safe('?');
-		if ($self->buffer =~ /^$r_VARNAME/) {
-			my $name	= $self->read_length($+[0]);
-			return $self->new_token(VAR, $self->start_line, $self->start_column, $name);
+		if (substr($self->buffer, 0, 1) eq '$') {
+			$self->get_char_safe('$');
+			if ($self->buffer =~ /^$r_VARNAME/) {
+				my $name	= $self->read_length($+[0]);
+				return $self->new_token(VAR, $self->start_line, $self->start_column, $name);
+			} else {
+				$self->_throw_error("Invalid variable name");
+			}
 		} else {
-			return $self->new_token(QUESTION, $self->start_line, $self->start_column, '?');
+			$self->get_char_safe('?');
+			if ($self->buffer =~ /^$r_VARNAME/) {
+				my $name	= $self->read_length($+[0]);
+				return $self->new_token(VAR, $self->start_line, $self->start_column, $name);
+			} else {
+				return $self->new_token(QUESTION, $self->start_line, $self->start_column, '?');
+			}
 		}
 	}
 	
@@ -425,6 +436,12 @@ package AtteanX::Parser::SPARQLLex::Iterator 0.021 {
 		} elsif (substr($buffer, 0, 2) eq '>=') {
 			$self->read_length(2);
 			return $self->new_token(GE, $self->start_line, $self->start_column, '>=');
+		} elsif (substr($buffer, 0, 2) eq '<<') {
+			$self->read_length(2);
+			return $self->new_token(LTLT, $self->start_line, $self->start_column, '<<');
+		} elsif (substr($buffer, 0, 2) eq '>>') {
+			$self->read_length(2);
+			return $self->new_token(GTGT, $self->start_line, $self->start_column, '>>');
 		} elsif (substr($buffer, 0, 1) eq '>') {
 			$self->get_char;
 			return $self->new_token(GT, $self->start_line, $self->start_column, '>');
@@ -480,6 +497,29 @@ package AtteanX::Parser::SPARQLLex::Iterator 0.021 {
 		}
 	}
 	
+	sub _get_brace_or_annotation_or_or {
+		my $self	= shift;
+		if (substr($self->buffer, 0, 2) eq '{|') {
+			$self->read_length(2);
+			return $self->new_token(LANNOT, $self->start_line, $self->start_column, '{|');
+		} elsif (substr($self->buffer, 0, 2) eq '|}') {
+			$self->read_length(2);
+			return $self->new_token(RANNOT, $self->start_line, $self->start_column, '|}');
+		} elsif (substr($self->buffer, 0, 2) eq '||') {
+			$self->read_length(2);
+			return $self->new_token(OROR, $self->start_line, $self->start_column, '||');
+		} elsif (substr($self->buffer, 0, 1) eq '{') {
+			$self->get_char_safe('{');
+			return $self->new_token(LBRACE, $self->start_line, $self->start_column, '{');
+		} elsif (substr($self->buffer, 0, 1) eq '}') {
+			$self->get_char_safe('}');
+			return $self->new_token(RBRACE, $self->start_line, $self->start_column, '}');
+		} else {
+			$self->get_char_safe('|');
+			return $self->new_token(OR, $self->start_line, $self->start_column, '|');
+		}
+	}
+
 	sub _get_lbracket_or_anon {
 		my $self	= shift;
 		if ($self->buffer =~ /^$r_ANON/) {
@@ -672,7 +712,7 @@ Gregory Todd Williams  C<< <gwilliams@cpan.org> >>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2014--2018 Gregory Todd Williams. This
+Copyright (c) 2014--2022 Gregory Todd Williams. This
 program is free software; you can redistribute it and/or modify it under
 the same terms as Perl itself.
 
