@@ -7,7 +7,7 @@ AtteanX::SPARQL::Token - Token objects used for parsing and serializing SPARQL
 
 =head1 VERSION
 
-This document describes AtteanX::SPARQL::Token version 0.033
+This document describes AtteanX::SPARQL::Token version 0.035
 
 =head1 SYNOPSIS
 
@@ -55,11 +55,11 @@ An array of values associated with the token (e.g. the integer value of an INT t
 
 =cut
 
-package AtteanX::SPARQL::Token 0.033;
+package AtteanX::SPARQL::Token 0.035;
 
 use Moo;
 use Types::Standard qw(ArrayRef Str);
-use List::MoreUtils qw(zip);
+use List::Util qw(mesh);
 use Sub::Util qw(set_subname);
 use AtteanX::SPARQL::Constants;
 use namespace::clean;
@@ -95,7 +95,7 @@ my @KEYS	= qw(type start_line start_column line column args);
 sub fast_constructor {
 	my $class = shift;
 	return $class->new(
-		zip @KEYS, @_
+		mesh \@KEYS, \@_
 	);
 }
 
