@@ -4,7 +4,7 @@ AtteanX::Serializer::SPARQLTSV - SPARQL Results TSV Serializer
 
 =head1 VERSION
 
-This document describes AtteanX::Serializer::SPARQLTSV version 0.033
+This document describes AtteanX::Serializer::SPARQLTSV version 0.035
 
 =head1 SYNOPSIS
 
@@ -35,13 +35,13 @@ This document describes AtteanX::Serializer::SPARQLTSV version 0.033
 use v5.14;
 use warnings;
 
-package AtteanX::Serializer::SPARQLTSV 0.033 {
+package AtteanX::Serializer::SPARQLTSV 0.035 {
 	use Moo;
 	use Types::Standard qw(Str ArrayRef);
 	use Encode qw(encode);
 	use Scalar::Util qw(blessed);
 	use Attean::ListIterator;
-	use List::MoreUtils qw(any);
+	use List::Util qw(any);
 	use namespace::clean;
 
 	has 'canonical_media_type' => (is => 'ro', isa => Str, init_arg => undef, default => 'text/tab-separated-values');
